package edu.uoc.tfg.learningpoint.business.courseadministration;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

import edu.uoc.tfg.learningpoint.business.useradministration.ITicket;
import edu.uoc.tfg.learningpoint.business.useradministration.TicketStore;
import edu.uoc.tfg.learningpoint.exception.LearningPointException;
import edu.uoc.tfg.learningpoint.exception.ValidationError;
import edu.uoc.tfg.learningpoint.integration.SellerJPA;
import edu.uoc.tfg.learningpoint.integration.StudentJPA;
import edu.uoc.tfg.learningpoint.integration.CourseJPA;

/**
 * EJB Session Bean Class of "Learningpoint"
 */
@Stateless
public class CourseAdministrationFacadeBean implements CourseAdministrationFacadeLocal, CourseAdministrationFacadeRemote, Serializable{

	/**
	 * SerialID
	 */
	private static final long serialVersionUID = -1040595550654633672L;
	
	
	@PersistenceContext
	private EntityManager entityManager;
	
	private TicketStore ticketStore = TicketStore.getTicketStore();
	
	/**
	 * Method that returns Collection of all sellers
	 */
	@Override
	public java.util.Collection<SellerJPA> listAllSellers( ITicket ticket ) throws LearningPointException {
		
		ticketStore.validateTicket( ticket );	
		
		final CriteriaBuilder cb = entityManager.getCriteriaBuilder();
		
		final CriteriaQuery<SellerJPA> cq = cb.createQuery(SellerJPA.class);
		
		@SuppressWarnings("unused")
		final Root<SellerJPA> sellerJPA = cq.from(SellerJPA.class);

		final TypedQuery<SellerJPA> query = entityManager.createQuery(cq);
	
		final List<SellerJPA> results = query.getResultList();
		
		return results;

	}
	
	/**
	 * Method that returns Collection of all courses by seller
	 */
	@Override
	public Collection<CourseJPA> listCoursesBySeller( ITicket ticket ) throws LearningPointException {
		
		ticketStore.validateTicket( ticket );	
		
		if ( ticket.isSeller() ){
		
			final SellerJPA sellerJPA = entityManager.find( SellerJPA.class, ticket.getNif() );
			
			return sellerJPA.getCourses();
		
		} else {
			System.err.println( String.format ( "%s : %s ", this.getClass(), "El usuario no es de tipo vendedor." ) );
			throw new CourseAdministrationException (new ValidationError("El usuario no es de tipo vendedor."));
		}
	    	  
	}
	
	
	/**
	 * Method that returns Collection of all courses: seller or student
	 */
	@Override
	public Collection<CourseJPA> listCourses( ITicket ticket ) throws LearningPointException {
		
		ticketStore.validateTicket( ticket );	
		
		if ( ticket.isSeller() ){
			
			final SellerJPA sellerJPA = entityManager.find( SellerJPA.class, ticket.getNif() );
			return sellerJPA.getCourses();
			
		} else {
			
			final StudentJPA studentJPA = entityManager.find( StudentJPA.class, ticket.getNif() );
			return studentJPA.getCourses();
			
		}
	    	  
	}
	
	
	/**
	 * Method that returns Collection of all students by course
	*/
	@Override
	public Collection<StudentJPA> listStudentsByCourse ( final ITicket ticket, final CourseJPA course ) throws LearningPointException {
			
		ticketStore.validateTicket( ticket );	
		
		final CourseJPA coursePersisted = entityManager.find( CourseJPA.class, course.getId() );
		
		return coursePersisted.getStudents();
	    	  
	}
	
	/**
	 * Method that returns instance of the class student
	 */
	@Override
	public StudentJPA showStudent ( final ITicket ticket ) throws LearningPointException {

		ticketStore.validateTicket( ticket );	
		
		if ( !ticket.isSeller() ) {
			
			final StudentJPA studentJPA = entityManager.find( StudentJPA.class, ticket.getNif() );
			return studentJPA;
			
		} else {
			System.err.println( String.format ( "%s : %s ", this.getClass(), "El usuario solicitado no es de tipo estudiante." ) );
			throw new CourseAdministrationException (new ValidationError("El usuario solicitado no es de tipo estudiante."));
			
		}

	}
	
	/**
	 * Method that returns instance of the class course
	 */
	@Override
	public CourseJPA showCourse ( final ITicket ticket, final CourseJPA courseJPA ) throws LearningPointException {

		ticketStore.validateTicket( ticket );	
		
		if ( courseJPA == null || courseJPA.getId() == null ){
			System.err.println( String.format ( "%s : %s ", this.getClass(), "No se puede recuperar un curso sin su identificador." ) );
			throw new CourseAdministrationException (new ValidationError("No se puede recuperar un curso sin su identificador."));
		}
		
		final CourseJPA courseJPAPersisted = entityManager.find (CourseJPA.class, courseJPA.getId() );
		
		return courseJPAPersisted;
		
	}
	
	/**
	 * Method that saves instance of the class course
	 */
	@Override
	public void addCourse ( final ITicket ticket, final CourseJPA courseJPA ) throws LearningPointException {
		
		ticketStore.validateTicket( ticket );	
		
		courseJPA.validateCaseInsert();
		
		if ( ticket.isSeller() ){
			
			SellerJPA sellerJPA = entityManager.find( SellerJPA.class, ticket.getNif() );
			sellerJPA.add (courseJPA);
			entityManager.persist(courseJPA);
			
		} else {
			System.err.println( String.format ( "%s : %s ", this.getClass(), "El usuario es tipo estudiante. No puede dar de alta un curso.") );
			throw new CourseAdministrationException (new ValidationError("El usuario es tipo estudiante. No puede dar de alta un curso."));
		}

				
	}
	
	/**
	 * Method that update instance of the class course
	 */
	@Override
	public void updateCourse( final ITicket ticket, final CourseJPA courseJPA) throws LearningPointException {

		ticketStore.validateTicket( ticket );	
		
		courseJPA.validateCaseUpdate();
		
		if ( ticket.isSeller() ){
			
			CourseJPA courseJPAStored = entityManager.find( CourseJPA.class, courseJPA.getId() );
			courseJPAStored.update(courseJPA);
			entityManager.merge(courseJPAStored);
			
		} else {
			System.err.println( String.format ( "%s : %s ", this.getClass(), "El usuario es tipo estudiante. No puede modificar un curso.") );
			throw new CourseAdministrationException (new ValidationError("El usuario es tipo estudiante. No puede modificar un curso."));
		}
		
	}
}
